@@ -5,7 +5,8 @@ use strict;
 use vars qw( $VERSION );
 $VERSION = '0.01';
 
-use CGI::Wiki::Plugin::RSS::ModWiki;
+use Wiki::Toolkit::Feed::Atom;
+use Wiki::Toolkit::Feed::RSS;
 use Time::Piece;
 use URI::Escape;
 use Carp 'croak';
@@ -22,8 +23,8 @@ sub _init {
 
     my $wiki = $args{wiki};
     
-    unless ( $wiki && UNIVERSAL::isa( $wiki, "CGI::Wiki" ) ) {
-       croak "No CGI::Wiki object supplied.";
+    unless ( $wiki && UNIVERSAL::isa( $wiki, "Wiki::Toolkit" ) ) {
+       croak "No Wiki::Toolkit object supplied.";
     }
     $self->{wiki} = $wiki;
 
@@ -59,24 +60,77 @@ sub make_feed {
     my ($self, %args) = @_;
     
     my $feed_type = $args{feed_type};
+    my $feed_listing = $args{feed_listing};
     
-    my %known_types = (
-                          'rss'  => 1,
-                      );
+    my %known_listings = (
+                          'recent_changes' => 1,
+                          'node_all_versions' => 1,
+                         );
                       
+    croak "No feed listing specified" unless $feed_listing;
+    croak "Unknown feed listing: $feed_listing" unless $known_listings{$feed_listing};
+
+    # Fetch the right Wiki::Toolkit::Feeds::Listing instance to use
+    my $maker = $self->fetch_maker($feed_type);
+
+    # Call the appropriate feed listing from it
+    if ($feed_listing eq 'recent_changes') {
+        return $maker->recent_changes(%args);
+    }
+    elsif ($feed_listing eq 'node_all_versions') {
+        return $maker->node_all_versions(%args);
+    }
+}
+
+=item B<fetch_maker>
+For the given feed type, identify and return the maker routine for feeds
+of that type. 
+
+my $maker = $feed->fetch_maker("rss");
+my $feed_contents = maker->node_all_versions(%options);
+
+Will always return something of type Wiki::Toolkit::Feed::Listing
+=cut
+sub fetch_maker {
+    my ($self,$feed_type) = @_;
+
+    my %known_types = (
+                          'atom'  => \&atom_maker,
+                          'rss' => \&rss_maker,
+                      );
+
     croak "No feed type specified" unless $feed_type;
     croak "Unknown feed type: $feed_type" unless $known_types{$feed_type};
 
-    if ($feed_type eq 'rss') {
-        return $self->rss_maker->recent_changes(%args);
+    return &{$known_types{$feed_type}};
+}
+
+sub atom_maker {
+    my $self = shift;
+  
+    unless ($self->{atom_maker}) {
+        $self->{atom_maker} = Wiki::Toolkit::Feed::Atom->new(
+            wiki                => $self->{wiki},
+            site_name           => $self->{site_name},
+            site_url            => $self->{config}->script_url,
+            site_description    => $self->{site_description},
+            make_node_url       => $self->{make_node_url},
+            recent_changes_link => $self->{config}->script_url . '?action=rc',
+            atom_link           => $self->{config}->script_url . '?action=rc&format=atom',
+            software_name       => 'OpenGuides',
+            software_homepage   => 'http://openguides.org/',
+            software_version    => $self->{og_version},
+        );
     }
+    
+    $self->{atom_maker};
 }
 
 sub rss_maker {
     my $self = shift;
 
     unless ($self->{rss_maker}) {
-        $self->{rss_maker} = CGI::Wiki::Plugin::RSS::ModWiki->new(
+        $self->{rss_maker} = Wiki::Toolkit::Feed::RSS->new(
             wiki                => $self->{wiki},
             site_name           => $self->{site_name},
             site_url            => $self->{config}->script_url,
@@ -95,8 +149,9 @@ sub rss_maker {
 sub feed_timestamp {
     my ($self, %args) = @_;
 
-    # The timestamp methods in our feeds are equivalent, we might as well
-    # use the RSS one.
+    # Call the compatability timestamping method on the RSS Feed.
+    # People should really just pass in also_return_timestamp to the
+    #  feed method, and get the timestamp at the same time as their data
     $self->rss_maker->rss_timestamp(%args);
 }
 
@@ -106,29 +161,32 @@ OpenGuides::Feed - generate data feeds for OpenGuides in various formats.
 
 =head1 DESCRIPTION
 
-Produces RSS 1.0 feeds for OpenGuides.  Distributed and 
+Produces RSS 1.0 and Atom 1.0 feeds for OpenGuides.  Distributed and 
 installed as part of the OpenGuides project, not intended for independent
 installation.  This documentation is probably only useful to OpenGuides
 developers.
 
 =head1 SYNOPSIS
 
-    use CGI::Wiki;
+    use Wiki::Toolkit;
     use OpenGuides::Config;
     use OpenGuides::Feed;
 
-    my $wiki = CGI::Wiki->new( ... );
+    my $wiki = Wiki::Toolkit->new( ... );
     my $config = OpenGuides::Config->new( file => "wiki.conf" );
     my $feed = OpenGuides::Feed->new( wiki       => $wiki,
                                       config     => $config,
                                       og_version => '1.0', ); 
 
     # Ten most recent changes in RSS format.
-    print "Content-Type: application/rdf+xml\n";
-    print "Last-Modified: " . $self->feed_timestamp( items => 10 ) . "\n\n";
     my %args = ( items     => 10,
-                 feed_type => 'rss', );
-    print $feed->make_feed( %args );
+                 feed_type => 'rss', 
+                 also_return_timestamp => 1 );
+    my ($feed_output,$feed_timestamp) = $feed->make_feed( %args );
+
+    print "Content-Type: application/rdf+xml\n";
+    print "Last-Modified: " . $feed_timestamp . "\n\n";
+    print $feed_output;
 
 =head1 METHODS
 
@@ -140,34 +198,60 @@ developers.
                                       config     => $config,
                                       og_version => '1.0', ); 
 
-C<wiki> must be a L<CGI::Wiki> object and C<config> must be an
+C<wiki> must be a L<Wiki::Toolkit> object and C<config> must be an
 L<OpenGuides::Config> object.  Both of these arguments are mandatory.
 C<og_version> is an optional argument specifying the version of
 OpenGuides for inclusion in the feed.
 
 =item B<rss_maker>
 
-Returns a raw L<CGI::Wiki::Plugin::RSS::ModWiki> object created with the values you
+Returns a raw L<Wiki::Toolkit::Feed::RSS> object created with the values you
+invoked this module with.
+
+=item B<atom_maker>
+
+Returns a raw L<Wiki::Toolkit::Feed::Atom> object created with the values you
 invoked this module with.
 
 =item B<make_feed>
 
     # Ten most recent changes in RSS format.
-    print "Content-Type: application/rdf+xml\n";
-    print "Last-Modified: " . $feed->feed_timestamp( items => 10 ) . "\n\n";
     my %args = ( items     => 10,
-                 feed_type => 'rss', );
+                 feed_type => 'rss',
+                 also_return_timestamp => 1 );
+    my ($feed_output,$feed_timestamp) = $rdf_writer->make_feed( %args );
+
+    print "Content-Type: application/rdf+xml\n";
+    print "Last-Modified: " . $feed_timestamp . "\n\n";
+    print $feed_output;
     print $rdf_writer->make_feed( %args );
 
+
+    # All the changes made by bob in the past week, ignoring minor edits, in Atom.
+    $args{days}               = 7;
+    $args{ignore_minor_edits  = 1;
+    $args{filter_on_metadata} => { username => "bob" };
+    $args{also_return_timestamp} => 1;
+
+    my ($feed_output,$feed_timestamp) = $rdf_writer->make_feed( %args );
+    print "Content-Type: application/atom+xml\n";
+    print "Last-Modified: " . $feed_timestamp . "\n\n";
+    print $feed_output;
+
 =item B<feed_timestamp>
 
+Instead of calling this, you should instead pass in the 'also_return_timestamp'
+option. You will then get back the feed timestamp, along with the feed output.
+
+This method will be removed in future, and currently will only return
+meaningful values if your arguments relate to recent changes.
+
     print "Last-Modified: " . $feed->feed_timestamp( %args ) . "\n\n";
 
-Returns the timestamp of the data feed in POSIX::strftime style ("Tue, 29 Feb 2000 
-12:34:56 GMT"), which is equivalent to the timestamp of the most recent item
-in the feed. Takes the same arguments as make_recentchanges_rss(). You will most 
-likely need this to print a Last-Modified HTTP header so user-agents can determine
-whether they need to reload the feed or not.
+Returns the timestamp of something in POSIX::strftime style ("Tue, 29 Feb 2000 
+12:34:56 GMT"). Takes the same arguments as make_recentchanges_rss(). 
+You will most likely need this to print a Last-Modified HTTP header so 
+user-agents can determine whether they need to reload the feed or not.
 
 =back
 
@@ -175,7 +259,7 @@ whether they need to reload the feed or not.
 
 =over 4
 
-=item * L<CGI::Wiki> and L<CGI::Wiki::Plugin::RSS::ModWiki>
+=item * L<Wiki::Toolkit>, L<Wiki::Toolkit::Feed::RSS> and L<Wiki::Toolkit::Feed::Atom>
 
 =item * L<http://openguides.org/>
 
